@@ -1,36 +1,42 @@
 from __future__ import annotations
 
-from typing import Any, Iterator, Dict
+from typing import Any, Optional, Iterator, Dict
 
 from collections.abc import Mapping, MutableMapping
 
-from copy import deepcopy
-
-from wysdom.mixins import RegistersSubclasses
-
 from ..exceptions import ValidationError
 from ..base_schema import SchemaAnything
 
 from .DOMElement import DOMElement
 from . import DOMInfo
 from .DOMProperties import DOMProperties
-from .functions import document
+from .functions import document, schema
 
 
-class DOMObject(DOMElement, MutableMapping, RegistersSubclasses):
+class DOMObject(DOMElement, MutableMapping):
+    """
+    An object with named properties.
+    """
 
     __json_schema_properties__: DOMProperties = None
-    __json_element_data__: Dict[str, DOMElement] = None
+    __json_element_data__: Dict[str, Optional[DOMElement]] = None
 
     def __init__(
             self,
             value: Mapping[str, Any] = None,
             json_dom_info: DOMInfo = None
     ) -> None:
+        """
+        :param value:         A dict (or any :class:`collections.abc.Mapping`) containing the data to populate this
+                              object's properties.
+        :param json_dom_info: A :class:`~wysdom.dom.DOMInfo` named tuple containing information about this object's
+                              position in the DOM.
+        """
         if value and not isinstance(value, Mapping):
             raise ValidationError(
                 f"Cannot validate input. Object is not a mapping: {value}"
             )
+        schema(self).validate(value)
         super().__init__(None, json_dom_info)
         self.__json_element_data__ = {}
         try:
@@ -39,27 +45,34 @@ class DOMObject(DOMElement, MutableMapping, RegistersSubclasses):
         except KeyError as e:
             raise ValidationError(str(e))
 
-    def __getitem__(self, key: str) -> DOMElement:
+    def __getitem__(self, key: str) -> Optional[DOMElement]:
         return self.__json_element_data__[key]
 
-    def __setitem__(self, key: str, value: DOMElement) -> None:
-        item_class = self.__json_schema_properties__.properties.get(
-            key, self.__json_schema_properties__.additional_properties)
-        if item_class is True:
-            item_class = SchemaAnything()
-        if not item_class:
-            raise KeyError(
-                f"No property named '{key}' exists, and "
-                "additional properties are not allowed."
+    def __setitem__(self, key: str, value: Optional[DOMElement]) -> None:
+        if value is None:
+            if key in self.__json_schema_properties__.required:
+                raise ValueError(
+                    f"The property '{key}' is not optional and cannot be None."
+                )
+            self.__json_element_data__[key] = None
+        else:
+            item_class = self.__json_schema_properties__.properties.get(
+                key, self.__json_schema_properties__.additional_properties)
+            if item_class is True:
+                item_class = SchemaAnything()
+            if not item_class:
+                raise KeyError(
+                    f"No property named '{key}' exists, and "
+                    "additional properties are not allowed."
+                )
+            self.__json_element_data__[key] = item_class(
+                value,
+                DOMInfo(
+                    document=document(self),
+                    parent=self,
+                    element_key=key
+                )
             )
-        self.__json_element_data__[key] = item_class(
-            value,
-            DOMInfo(
-                document=document(self),
-                parent=self,
-                element_key=key
-            )
-        )
 
     def __delitem__(self, key: str) -> None:
         del self.__json_element_data__[key]
@@ -77,6 +90,7 @@ class DOMObject(DOMElement, MutableMapping, RegistersSubclasses):
         return str(self.__json_element_data__)
 
     def walk_elements(self) -> Iterator[DOMInfo]:
+        yield self.__json_dom_info__
         for key, value in self.items():
             if isinstance(value, DOMElement):
                 yield from value.walk_elements()
@@ -100,10 +114,16 @@ class DOMObject(DOMElement, MutableMapping, RegistersSubclasses):
 
     def __copy__(self) -> DOMObject:
         cls = self.__class__
-        return cls(dict(self))
+        result = cls.__new__(cls)
+        super(DOMObject, result).__init__(json_dom_info=self.__json_dom_info__)
+        result.__json_element_data__ = dict(self.__json_element_data__)
+        return result
 
     def __deepcopy__(self, memo: Dict[int, DOMElement]) -> DOMObject:
         cls = self.__class__
-        result = cls(deepcopy(dict(self), memo))
+        result = cls(
+            value=self.to_builtin(),
+            json_dom_info=self.__json_dom_info__
+        )
         memo[id(self)] = result
         return result
